#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "rpc.h"
#include "rpc_service.h"

extern int	verbose;

/*
 * this is the kind of code that might be generated by
 * an RPC generator, which takes a description of the RPC
 * methods (return type, argument types), and generate
 * code which implements the client and server side code.
 *
 * this is the server side code.
 */
void rpc_hndlr0(struct srss_state *sp) {
	int	i0, i1, i2;
	double	d0, d1;

	/*
	 * unmarshall the input arguments from the remote caller.
	 */
	if (srss_unmarshall(sp,"dlfdlfd",&i0,&d0,&i1,&d1,&i2) != 5) {
		fprintf(stderr,"rpc_hndlr0: arg count wrong\n");
		return;
	}
	if (verbose) {
		printf("rpc_hndlr0: unmarshalled\n");
		printf("i0 = %d\n",i0);
		printf("i1 = %d\n",i1);
		printf("i2 = %d\n",i2);
		printf("d0 = %f\n",d0);
		printf("d1 = %f\n",d1);
	}
	/*
	 * typically an RPC stub generator would have code here which
	 * simply invokes the programmer-supplied function.
	 *
	 * here we just have it in-lined.
	 */
	d0 += d1 + i0 + i1 + i2;
	/*
	 * out arguments and return values are marshalled to be sent
	 * back to the remote caller.
	 */
	if (verbose) {
		printf("rpc_hndlr0: returning d0 = %f\n",d0);
	}
	srss_marshall(sp,"lf",d0);
}

/*
 * second RPC.
 */
void rpc_hndlr1(struct srss_state *sp) {
	int	i0, i1;
	double	d0, d1;

	if (srss_unmarshall(sp,"dlfdlf",&i0,&d0,&i1,&d1) != 4) {
		fprintf(stderr,"rpc_hndlr1: arg count wrong\n");
		return;
	}
	if (verbose) {
		printf("rpc_hndlr1: unmarshalled\n");
		printf("i0 = %d\n",i0);
		printf("i1 = %d\n",i1);
		printf("d0 = %f\n",d0);
		printf("d1 = %f\n",d1);
	}
	d0 += d1 - i0 - i1;
	if (verbose) {
		printf("rpc_hndlr1: returning d0 = %f\n",d0);
	}
	srss_marshall(sp,"lf",d0);
}

/*
 * list of all RPCs exported.
 */
struct srss_entry entries[] = {
	{ "RPC 0", rpc_hndlr0, },
	{ "RPC 1", rpc_hndlr1, },
};

int start_service(int sock)
{
	struct srss_state	ifc;
	
	srss_server_init(&ifc,sizeof entries/sizeof entries[0],entries,sock);
	return srss_service(&ifc,1);
}

void rpc_service(int	sock)
{
	int			rv;

	if (verbose)
		fprintf(stderr,"child starting service, pid %lu\n",(unsigned long) getpid());
	rv = start_service(sock);
	if (verbose)
		fprintf(stderr,"child service routine returned %d\n",rv);
}
